﻿local GCD = LibStub("AceAddon-3.0"):NewAddon("GCD", "AceConsole-3.0", "AceEvent-3.0")

local GetTime = GetTime

local GCDBar ,GCDSpark
local starttime, duration, spellid, warned

local BOOKTYPE_SPELL = BOOKTYPE_SPELL

GCD.options = {
	desc = "GCD",
	type = "group",
	args = {
		spell = {
			name = "spell",
			desc = "Set Spell to track GCD",
			type = "input",
			set = function(info,val)
				GCD.db.profile.spell = strlower(val)
				GCD:SPELLS_CHANGED() end,
		},
		widescreen = {
			name = "widescreen",
			desc = "Set True if using widescreen",
			type = "toggle",
			get = function(info) return GCD.db.profile.widescreen end,
			set = function(info,val) GCD.db.profile.widescreen = val GCD:Print("Widescreen set to: "..tostring(val)) end,
		},
	},
}

local function OnUpdate(self, elapsed)
	local x, y = GetCursorPosition();
	GCDBar:ClearAllPoints()
	GCDBar.texture:ClearAllPoints()
	GCDBar.spark:ClearAllPoints()
	if GCD.db.profile.widescreen then
		GCDBar:SetPoint('CENTER', UIParent, 'BOTTOMLEFT', x*1.11+10, y*1.11-10)
	else
		GCDBar:SetPoint('CENTER', UIParent, 'BOTTOMLEFT', x+10, y-10)
	end
	GCDBar.texture:SetPoint('CENTER', GCDBar, 'CENTER')
	
 	GCDBar.texture.timer = GCDBar.texture.timer + elapsed;
 	if ( GCDBar.texture.timer > 0.02 ) then
  		GCDBar.texture.hAngle = GCDBar.texture.hAngle + 0.5;
  		GCDBar.texture.s = sin(GCDBar.texture.hAngle);
  		GCDBar.texture.c = cos(GCDBar.texture.hAngle);
  		GCDBar.texture:SetTexCoord(	0.5-GCDBar.texture.s, 0.5+GCDBar.texture.c,
  					0.5+GCDBar.texture.c, 0.5+GCDBar.texture.s,
  					0.5-GCDBar.texture.c, 0.5-GCDBar.texture.s,
  					0.5+GCDBar.texture.s, 0.5-GCDBar.texture.c);
  		GCDBar.texture.timer = 0;
 	end

	
	local perc = (GetTime() - starttime) / duration
	if perc > 1 then
		return GCDBar:Hide()
	else
		local angle = math.rad(90-perc*360)
		local x, y = math.cos(angle), math.sin(angle)
		GCDBar.spark:SetPoint('CENTER', GCDBar, 'CENTER',x*25,y*25)
	end
end

local function OnHide()
	GCDBar:SetScript('OnUpdate', nil)
end

local function OnShow()
	GCDBar:SetScript('OnUpdate', OnUpdate)
end

local defaults = {
	char = {
		spell = '',
		widescreen = false,
	}
}

function GCD:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("GCDDB", defaults, "char")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("GCD", self.options)
	LibStub("AceConfigCmd-3.0"):CreateChatCommand("gcd", "GCD")
end

function GCD:OnEnable()
	self:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN")
	self:RegisterEvent("SPELLS_CHANGED")
	if not GCDBar then
		GCDBar = CreateFrame('Frame', 'GCDBar', UIParent)
		GCDBar:ClearAllPoints()
		GCDBar:SetFrameStrata('HIGH')
		GCDBar:SetScript('OnShow', OnShow)
		GCDBar:SetScript('OnHide', OnHide)
		GCDBar:SetWidth(64)
		GCDBar:SetHeight(64)
		GCDBar:Hide()
		
		GCDBar.texture = GCDBar:CreateTexture(nil, 'ARTWORK')
		GCDBar.texture:SetTexture("Spells\\AURARUNE8")
		GCDBar.texture:SetVertexColor(0,1,0,0.5)
		GCDBar.texture:SetBlendMode('ADD')
		GCDBar.texture:SetWidth(80)
		GCDBar.texture:SetHeight(80)
		GCDBar.texture.timer = 0;
		GCDBar.texture.hAngle = 0;
		
		GCDBar.spark = GCDBar:CreateTexture(nil, 'DIALOG')
		GCDBar.spark:SetTexture("Spells\\AURA_01")
		GCDBar.spark:SetVertexColor(1,1,1)
		GCDBar.spark:SetBlendMode('ADD')
		GCDBar.spark:SetWidth(30)
		GCDBar.spark:SetHeight(30)
	end
	
	self:SPELLS_CHANGED()
end

function GCD:OnDisable()
	GCDBar:Hide()
	self:UnregisterAllEvents()
	GCDSpark = nil
	GCDBar = nil
end

function GCD:ACTIONBAR_UPDATE_COOLDOWN()
	if spellid then
		local start, dur = GetSpellCooldown(spellid, BOOKTYPE_SPELL)
		if dur > 0 and dur <= 2 then
			starttime = start
			duration = dur
			GCDBar:Show()
			return
		elseif dur == 0 then
			GCDBar:Hide()
		end
	end
end

function GCD:SPELLS_CHANGED()
	if GCDBar then
		if self.db.profile.spell == '' and not warned then
			self:Print("Kein Spell gesetzt!")
			warned = true
		else
			spellid = nil
			for tab = 1, 4 do
				local _, _, offset, numSpells = GetSpellTabInfo(tab)
				for i = (1+offset), (offset+numSpells) do
					local spell = GetSpellName(i, BOOKTYPE_SPELL)
					if strlower(spell) == self.db.profile.spell then
						spellid = i
						break
					end
				end
			end
		end
	end
end
